#!/usr/bin/python

"""
Author: Honza Horak <hhorak@redhat.com>
License: BSD

This program opens passwd.byuid NIS map for specified domainname
and checks if there are some UIDs between 500 and 1000.
If so, these could make troubles after upgrading.
Exit value is 0 when there are suspicious UIDs, 1 if there are
no suspicious UIDs, higher value if some error occured.
"""

import sys, gdbm


def uid_is_dangerous(uid):
    if not uid:
        return False

    try:
        uid_num = int(uid)
        if uid_num >= 500 and uid_num < 1000:
            return True
    except ValueError:
        pass
    return False


def print_help():
    print ("Checks if passwd NIS map includes UID lower than 1000 for a domainname")
    print ("Usage: %s domainname")


def check_uids():
    if len(sys.argv) < 2:
        print_help()
        return (3)

    domainname=sys.argv[1]
    if domainname == "":
        return (2)

    map_file = "/var/yp/%s/passwd.byuid" % domainname
    try:
        fh = gdbm.open(map_file, 'r')
        if not fh:
            return (2)
    except gdbm.error as e:
        sys.stderr.write("Cannot open NIS map file %s for reading: %s" % (map_file, e))
        return (2)

    uid=fh.firstkey()
    while uid:
        uid=fh.nextkey(uid)
        if uid_is_dangerous(uid):
            return 0

    fh.close()
    return 1


if __name__ == "__main__":
    exit(check_uids())




#!/usr/bin/python
# -*- Mode: Python; python-indent: 8; indent-tabs-mode: t -*-

import sys
import os
from preup.script_api import *
import ConfigParser

"""Preupgrade Assistant performs system upgradability assessment
and gathers information required for successful operating system upgrade.
Copyright (C) 2013 Red Hat Inc.
Petr Hracek <phracek@redhat.com>

This program is free software: you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation, either version 3 of the License, or
(at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program.  If not, see <http://www.gnu.org/licenses/>."""
set_component("distribution")
#END GENERATED SECTION


def write_file(file_name, data):
    fd = open(file_name, 'w')
    fd.writelines(data)
    fd.close()


def read_file(file_name):
    fd = open(file_name, 'r')
    lines = fd.readlines()
    fd.close()
    return lines


class HandlingRepoFiles(object):

    def __init__(self, dirname):
        self.dirname = dirname
        self.repo_files = []
        self.repo_dict = {}

    def find_repo_files(self):
        for repo_file in os.listdir(self.dirname):
            if repo_file.endswith('.repo'):
                self.repo_files.append(os.path.join(self.dirname, repo_file))

    def update_dict(self, cfp, section, option):
        if cfp.has_option(section, option):
            #if section not in self.repo_dict:
            #    self.repo_dict[section] = {}
            self.repo_dict[section] = '%s=%s' % (option, cfp.get(section, option))
            #self.repo_dict[section][option] = cfp.get(section, option)

    def check_repos(self):
        for repo_file in self.repo_files:
            cfp = ConfigParser.ConfigParser()
            cfp.readfp(open(repo_file))
            for section in cfp.sections():
                options = cfp.options(section)
                try:
                    if not cfp.has_option(section, 'enabled') or int(cfp.get(section, 'enabled')):
                        log_slight_risk("Repo %s is enabled." % section)
                        self.update_dict(cfp, section, 'mirrorlist')
                        self.update_dict(cfp, section, 'metalink')
                        self.update_dict(cfp, section, 'baseurl')
                    else:
                        log_slight_risk("Repo %s is not enabled." % section)
                except ConfigParser.NoOptionError:
                    continue
                except TypeError:
                    continue

    def save_relevant_repos(self):
        dir_name = os.path.join(VALUE_TMP_PREUPGRADE, 'kickstart')
        file_name = 'available-repos'
        write_file(os.path.join(dir_name, file_name),
                   '\n'.join([key+'='+value for key, value in self.repo_dict.iteritems()]))
        lines = read_file(os.path.join(dir_name, file_name))
        write_file(os.path.join(dir_name, file_name), lines)
        log_slight_risk('Enabled repo files for kickstart generation are stored %s.' % os.path.join(dir_name, file_name))


def main():
    hrf = HandlingRepoFiles('/etc/yum.repos.d')
    hrf.find_repo_files()
    hrf.check_repos()
    hrf.save_relevant_repos()
    exit_fail()


if __name__ == "__main__":
    #set_component('yum')
    main()



#!/usr/bin/python
# -*- Mode: Python; python-indent: 8; indent-tabs-mode: t -*-

import sys, os, errno
import datetime
import re

devel_time = False
if devel_time:
	def log_debug(x):
		print x

        def log_slight_risk(x):
		print x

        def log_warning(x):
		print x

	def log_error(x):
		print x

	def exit_pass():
		sys.exit(0)

	def exit_fail():
		sys.exit(1)

	def exit_error():
		sys.exit(-1)
else:
	from preup.script_api import *

"""Preupgrade Assistant performs system upgradability assessment
and gathers information required for successful operating system upgrade.
Copyright (C) 2013 Red Hat Inc.
Frantisek Kluknavsky <fkluknav@redhat.com>

This program is free software: you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation, either version 3 of the License, or
(at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program.  If not, see <http://www.gnu.org/licenses/>."""
check_rpm_to (check_rpm="filesystem",check_bin="")
set_component("distribution")
#END GENERATED SECTION

# exit functions are exit_{pass,not_applicable, fixed, fail, etc.}
# logging functions are log_{error, warning, info, etc.}
# for logging in-place risk use functions log_{extreme, high, medium, slight}_risk

def cgroup_daemon():
	cgroup_re = re.compile(r"\bCGROUP_DAEMON\b") #\b  Matches the empty string, but only at the beginning or end of a word. A word is defined as a sequence of alphanumeric or underscore characters,
	found = False
	for root, dirs, files in os.walk('/etc/sysconfig'):
		for filename in files:
			with open(root+"/"+filename, "r") as f:
				data = f.read()
			if cgroup_re.search(data):
				found = True
				log_slight_risk("File '" + filename + "' mentions CGROUP_DAEMON.")

	return found


if __name__ == "__main__":
	if cgroup_daemon():
		exit_fail()
	else:
		exit_pass()



#!/usr/bin/python
# -*- Mode: Python; python-indent: 8; indent-tabs-mode: t -*-

import sys
import os
import re

from preup.script_api import *


"""Preupgrade Assistant performs system upgradability assessment
and gathers information required for successful operating system upgrade.
Copyright (C) 2013 Red Hat Inc.
Frantisek Kluknavsky <fkluknav@redhat.com>

This program is free software: you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation, either version 3 of the License, or
(at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program.  If not, see <http://www.gnu.org/licenses/>."""
check_applies_to (check_applies="filesystem")
set_component("filesystem")
#END GENERATED SECTION
component = "vcsrepos"
# exit functions are exit_{pass,not_applicable, fixed, fail, etc.}
# logging functions are log_{error, warning, info, etc.}
# for logging in-place risk use functions log_{extreme, high, medium, slight}_risk

def get_file_content(filename):
    f = open(filename, "r")
    lines = f.readlines()
    f.close()
    return lines

def update_solution_file(path, suffix):
    solution_file(path[:-len(suffix)]+"\n")

def git_repository(path, suffix):
    try:
        gitconfig = get_file_content(os.path.join(path, "config"))
        if gitconfig:
            update_solution_file(path, suffix)
            for line in gitconfig:
                url = re.match(r"\s*url\s*=\s*(.*)", line)
                if url:
                    addr = re.match(r"(ftp://|https?://)(.*)", url.group(1))
                    if addr:
                        solution_file("url = [link: " + url.group(1) + "]\n")
                    else:
                        solution_file(line)
    except IOError:
        pass


def bzr_repository(path, suffix):
    try:
        config = get_file_content(os.path.join(path, "branch", "branch.conf"))
        if config:
            update_solution_file(path, suffix)
            for line in config:
                addr = re.match(r"\s*parent_location\s*=\s*(.*)", line)
                if addr:
                    if re.match(r"(https?|ftp)://", addr.group(1)):
                        solution_file("url = [link: " + addr.group(1) + "]\n")
                    else:
                        solution_file("url = " + addr.group(1) + "\n")
    except IOError:
        pass


def svn_repository(path, suffix):
    try:
        config = get_file_content(os.path.join(path, "entries"))
        if config:
            update_solution_file(path, suffix)
            line = config[4]
            if re.match(r"\s*(https?|ftp)://", line):
                solution_file("url = [link: " + line[:-1] + "]\n")
            else:
                solution_file("url = " + line)
    except IOError:
        pass

def cvs_repository(path, suffix):
    try:
        config = get_file_content(os.path.join(path, "Root"))
        if config:
            update_solution_file(path, suffix)
            for line in config:
                solution_file(line)
    except IOError:
        pass


def main():
    if os.geteuid() != 0:
        sys.stdout.write("Needs to be root.\n")
        log_slight_risk("The script needs to be run under the root account")
        exit_error()
    messages = {'/.git': "GIT", '/.svn': "SUBVERSION", '/.bzr': "BAZAAR", '/CVS': "CVS"}
    fnc_dict = {'/.git': git_repository,
                '/.svn': svn_repository,
                '/.bzr': bzr_repository,
                '/CVS': cvs_repository}
    homefiles = get_file_content(VALUE_ALLMYFILES)
    paths = filter(lambda x: x.startswith("/home/") or x.startswith("/root"), homefiles)
    if paths:
        for suffix in fnc_dict.iterkeys():
            repo_paths = filter(lambda x: x.strip().endswith(suffix), paths)
            if repo_paths:
                solution_file("Found %s repositories\n" % messages[suffix])
                for path in paths:
                    path = path.strip()
                    try:
                        fnc_dict[suffix](path, suffix)
                    except Exception:
                        solution_file(path[:-len(suffix)]+"\n")
                solution_file("\n")
    else:
        solution_file("VCS repositories not found.")


if __name__ == "__main__":
    set_component(component)
    main()
    exit_informational()



#!/usr/bin/python

import subprocess
import re
from preup.script_api import *


"""Preupgrade Assistant performs system upgradability assessment
and gathers information required for successful operating system upgrade.
Copyright (C) 2013 Red Hat Inc.
John Dennis <jdennis@redhat.com>

This program is free software: you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation, either version 3 of the License, or
(at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program.  If not, see <http://www.gnu.org/licenses/>."""
check_applies_to (check_applies="freeradius")
check_rpm_to (check_rpm="python",check_bin="")
set_component("freeradius")
#END GENERATED SECTION

RADIUSD = '/usr/sbin/radiusd'
COMPONENT = 'freeradius'

def get_int(string):
    try:
        value = int(string)
        return value
    except ValueError:
        match = re.search(r'\d+', string)
        if match:
            value = int(match.group(0))
            return value
        else:
            raise ValueError("cannot find integer in '%s'" % string)

def get_radiusd_version():
    args = [RADIUSD, '-v']
    p = subprocess.Popen(args, stdout=subprocess.PIPE, stderr=subprocess.PIPE)
    stdout, stderr = p.communicate()
    returncode = p.returncode
    if returncode != 0:
        raise subprocess.CalledProcessError(returncode=returncode,
                                            cmd=' '.join(args),
                                            output=stderr)

    match = re.search(r'FreeRADIUS\s+Version\s+(\d+)\.(\d+)\.(\d+)(\S*?),', stdout, re.MULTILINE)
    if match:
        major = match.group(1)
        minor = match.group(2)
        micro = match.group(3)
        extra = match.group(4)
    else:
        raise ValueError("unable to parse FreeRADIUS version")

    return major, minor, micro, extra

def main():
    try:
        major, minor, micro, extra = get_radiusd_version()
    except Exception as e:
        log_error("Unable to query FreeRADIUS version: %s" % e)
        exit_error()

    log_info("found FreeRADIUS version %s.%s.%s%s" % (
        major, minor, micro, extra))

    try:
        major = get_int(major)
        minor = get_int(minor)
        micro = get_int(micro)
    except Exception as e:
        log_error("Unable to determine numeric version values: %s" % e)
        exit_error()


    if major < 3:
        log_high_risk("The configuration of FreeRadius %d is not compatible with"
                      " version 3 in Red Hat Enterprise Linux 7. See remediation"
                      " description."  % (major))
        exit_fail()

if __name__ == "__main__":
    set_component(COMPONENT)
    main()
    exit_pass()



#!/usr/bin/python
# -*- Mode: Python; python-indent: 8; indent-tabs-mode: t -*-

import sys, os #, errno
import subprocess
import re

devel_time = False
if devel_time:
	def log_debug(x):
		print x

	def log_warning(x):
		print x

	def log_error(x):
		print x

	def exit_pass():
		sys.exit(0)
	
	def exit_fail():
		sys.exit(1)

	def exit_error():
		sys.exit(-1)
else:
	from preup.script_api import *

"""Preupgrade Assistant performs system upgradability assessment
and gathers information required for successful operating system upgrade.
Copyright (C) 2013 Red Hat Inc.
Lukas Nykryn <lnykryn@redhat.com>

This program is free software: you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation, either version 3 of the License, or
(at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program.  If not, see <http://www.gnu.org/licenses/>."""
check_applies_to (check_applies="initscripts")
set_component("initscripts")
#END GENERATED SECTION
# exit functions are exit_{pass,not_applicable, fixed, fail, etc.}
# logging functions are log_{error, warning, info, etc.}
# for logging in-place risk use functions log_{extreme, high, medium, slight}_risk

def ifcfg_error():
	"""https://bugzilla.redhat.com/show_bug.cgi?id=1111174
1) kernel names (eth0, eth1, ...) 
a) DEVICE + HWADDR specified in ifcfg files = not safe, udev in rhel7 is not able to swap names (onlky exception here are machines with only one network interface, they should be fine) <- this is only thing which is covered by content which I wrote
b) only DEVICE specified in ifcfg files = not safe, udev will probably name the interface differently in rhel7
c) only HWADRR specified in ifcfg files = uh, device will be set up correctly according to ifcfg file but name could be different
2) biosdevname names (em1, p3p4, p3p4_1, ...)
On dell machines you should be fine in all cases, for others biosdevname was not turned on by default, but it should work.
3) custom names (my_little_network_card, ...)
both HWADDR and DEVICE are specified in this case or you have written your own udev rules.
This is safe in the case that you use truly unique names, they should not match any other naming methods (kernel, biosdevname or udev in rhel7).
1a is covered ()
We should refuse to update also in the 1b case.
If you are using 1c case you are used to some level of pain, so it would be enough just to write a warning. 
2 is fine.
I case 3, to be completely correct you could check that user is not using rhel7 udev naming scheme [1] in rhel6, but I would bet that nobody uses that.
"""


	ifcfg_path = "/etc/sysconfig/network-scripts/"
	ifcfg_prefix = "ifcfg-"

	def get_variable(path, variable):
		err_msg = "Error while reading ifcfg scripts"
		def err(output):
			log_error(err_msg)
			log_error(output.read())

		try:
			output = os.tmpfile();
			(currentpath, _ ) = os.path.split(os.path.realpath(__file__))
			return_code = subprocess.Popen([currentpath + "/get_var_by_name.sh", path, variable],
			bufsize=0,
			executable=None,
			stdin=None,
			stdout=output,
			stderr=subprocess.STDOUT,
			preexec_fn=None,
			close_fds=True,
			shell=False,
			cwd=None,
			env=None,
			universal_newlines=False,
			startupinfo=None,
			creationflags=0).wait()
		except:
			output.seek(0)
			err(output)
			raise
		output.seek(0)
		if return_code < 0:#error
			log_error(output.read())
			raise err_msg
		if return_code == 1:#undefined variable
			return False
		return output.read().strip()

	def ls_scripts():
		for f in [i for i in os.listdir(ifcfg_path) if i.startswith(ifcfg_prefix)]:
			yield f
	
	def is_kernel(name):
		return re.match("eth[0-9]+", name)

	def is_udev(name):
		if name[0:2] in ("en", "sl", "wl", "ww"):
			if name[2] in ("b", "c", "o", "s", "x", "P", "p"):
				return True
		return False


	ethx_with_addr_count = 0
	warning = False
	for script in ls_scripts():
		full_path = ifcfg_path + script
		addr = get_variable(full_path, "HWADDR")
		name = get_variable(full_path, "NAME" if devel_time else "DEVICE")
		log_debug("checking " + script + ", name: " + str(name) + " hwaddr: " + str(addr))
		short_name = script[len(ifcfg_prefix):]
		if short_name == "lo": #loopback
			continue
		if not name:
			warning = True
			if not addr:
				log_slight_risk(full_path + " does not have DEVICE nor HWADDR set, what kind of device it is?")
			else:
				log_slight_risk(full_path + " does not have DEVICE set, its name can change after upgrade.")
		else:
			if is_kernel(name):
				if addr:
					ethx_with_addr_count +=1
				else:
					log_slight_risk(full_path + " is old style ethX name without HWADDR, its name can change after upgrade.")
					warning = True
			elif is_udev(name):
				log_slight_risk(full_path + " variable DEVICE is very similar to udev predictable network naming scheme, it may cause conflicts.")
				warning = True

		if ethx_with_addr_count > 1:
			log_medium_risk("You use multiple network devices with old style 'ethX' names.")
			warning = True
		if ethx_with_addr_count == 1:
			log_slight_risk("You use one network device with old style 'ethX' name. This will work as long as you only have one but will break with multiple such devices.")
			warning = True
	return warning

if __name__ == "__main__":
	if os.geteuid() != 0 and not devel_time:
		sys.stdout.write("Need to be root.\n")
		log_slight_risk("The script needs to be run under root account")
		exit_error()
	if ifcfg_error():
		exit_fail()
	else:
		exit_pass()



#!/usr/bin/python
# -*- Mode: Python; python-indent: 8; indent-tabs-mode: t -*-

from preup.script_api import *
component = "filesystem"

set_component("filesystem")
"""Preupgrade Assistant performs system upgradability assessment
and gathers information required for successful operating system upgrade.
Copyright (C) 2013 Red Hat Inc.
Frantisek Kluknavsky <fkluknav@redhat.com>

This program is free software: you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation, either version 3 of the License, or
(at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program.  If not, see <http://www.gnu.org/licenses/>."""
check_applies_to (check_applies="filesystem")
check_rpm_to (check_rpm="",check_bin="lsblk")
set_component("filesystem")
#END GENERATED SECTION

import sys, os
import subprocess

# exit functions are exit_{pass,not_applicable, fixed, fail, etc.}
# logging functions are log_{error, warning, info, etc.}
# for logging in-place risk use functions log_{extreme, high, medium, slight}_risk



def find_removable():
	"""ask lsblk for removable devices and warn if they are in fstab without 'nofail'"""
	
	def nofail_flag(fstab_flags):
		fstab_flags = fstab_flags.split(",")
		for flag in fstab_flags:
			if flag == "nofail":
				return True			
		return False

	def run_command(cmd):
		"""run external command specified as a list of argv, return status code and output as a list of lines"""
		output = os.tmpfile()
		try:
			return_code = subprocess.Popen(cmd,
							stdout=output,
							stderr=subprocess.STDOUT,
							close_fds=True,
							shell=False,
							creationflags=0).wait()
		except:
			output.seek(0)
			log_error(output.read())
			log_error("Could not invoke external command " + str(cmd))
			exit_error()
		output.seek(0)
		return (return_code, output.read())
	
	def lsblk_removables():
		"""ask lsblk, return list of removable media"""
		#lsblk in rhel 6 does not have --paths option ;-(
		cmd = ["lsblk", "--all", "--list", "--output", "NAME,RM"]
		(return_code, output) = run_command(cmd)
		if return_code !=  0:
			log_error("lsblk failed")
			log_error(output)
			exit_fail()
		output = output.split('\n')
		ret = []
		for line in output:
			tmp=line.split()
			#mind possible empty line and/or multiword name
			if tmp and tmp[len(tmp)-1] == "1":
				ret.append(tmp[0])
		return ret


	ret = False
	lsblk_list = lsblk_removables()
	with open(r"/etc/fstab", "r") as fstab_tmp:
		fstab = []
		for line in fstab_tmp.readlines():
			fstab.append(line.split())
	for device in lsblk_list:
		for fstab_line in fstab:
			#lsblk in rhel6 does not give full path, search for a substring corresponding to partial path
			if fstab_line and fstab_line[0].find(device) != -1  and not nofail_flag(fstab_line[3]):
				log_extreme_risk("It seems that you have a removable device " + device + " in /etc/fstab entry " + fstab_line[0] + " without nofail flag")
				ret = True
	return ret

if __name__ == "__main__":
	if os.geteuid() != 0:
		log_error("The script needs to be run under root account")
		exit_error()
	if find_removable():
		exit_fail()
	else:
		exit_informational()



#!/usr/bin/python
# -*- Mode: Python; python-indent: 8; indent-tabs-mode: t -*-

from preup.script_api import *
component = "filesystem"

set_component("filesystem")
"""Preupgrade Assistant performs system upgradability assessment
and gathers information required for successful operating system upgrade.
Copyright (C) 2013 Red Hat Inc.
Frantisek Kluknavsky <fkluknav@redhat.com>

This program is free software: you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation, either version 3 of the License, or
(at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program.  If not, see <http://www.gnu.org/licenses/>."""
check_applies_to (check_applies="filesystem")
set_component("filesystem")
#END GENERATED SECTION

import sys, os
import re

# exit functions are exit_{pass,not_applicable, fixed, fail, etc.}
# logging functions are log_{error, warning, info, etc.}
# for logging in-place risk use functions log_{extreme, high, medium, slight}_risk

def ro_dirs(paths):
	def ro_flag(mtab_line):
		mtab_flags = mtab_line[3].split(",")
		for flag in mtab_flags:
			if flag == "ro":
				return True			
		return False

	class FoundRO(Exception):
		pass

	log_info("Checking if /etc/mtab contains paths: " + str(paths))
	ret = False
	with open(r"/etc/mtab", "r") as mtab_tmp:
		mtab = []
		for line in mtab_tmp.readlines():
			mtab.append(line.split())
	for path in paths:
		try:
			for mtab_line in mtab:
				path_prefix = path
				while path_prefix:
					if path_prefix == mtab_line[1]:
						if ro_flag(mtab_line):
							ret = True
							log_extreme_risk(mtab_line[1]+" is read-only. In-place upgrade requires "+path+" to be writable!")
							raise FoundRO()
					path_prefix = path_prefix[:path_prefix.rindex(r"/")]
		except FoundRO:
			pass	

		#check root separately
	for mtab_line in mtab:
		if ro_flag(mtab_line):
			ret = True
			log_medium_risk("Mount point "+mtab_line[1]+" is mounted read-only.")
	return ret

if __name__ == "__main__":
	if os.geteuid() != 0:
		log_error("The script needs to be run under root account")
		exit_error()
	ro_dirs_result = ro_dirs((r"/usr", r"/var", r"/var/run", r"/var/lock"))
	if ro_dirs_result:
		#log_extreme_risk("Found crucial directories mounted read-only: "+ro_dirs_result)
		exit_informational()
	else:
		exit_pass()



#!/usr/bin/python
# -*- Mode: Python; python-indent: 8; indent-tabs-mode: t -*-

import sys, os #, errno
#import datetime
import subprocess
import re

from preup.script_api import *


"""Preupgrade Assistant performs system upgradability assessment
and gathers information required for successful operating system upgrade.
Copyright (C) 2013 Red Hat Inc.
Frantisek Kluknavsky <fkluknav@redhat.com>

This program is free software: you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation, either version 3 of the License, or
(at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program.  If not, see <http://www.gnu.org/licenses/>."""
check_applies_to (check_applies="hal")
check_rpm_to (check_rpm="",check_bin="rpm")
set_component("hal")
#END GENERATED SECTION
# exit functions are exit_{pass,not_applicable, fixed, fail, etc.}
# logging functions are log_{error, warning, info, etc.}
# for logging in-place risk use functions log_{extreme, high, medium, slight}_risk

def splitFilename(filename):
    """ 
    Pass in a standard style rpm fullname 
    
    Return a name, version, release, epoch, arch, e.g.::
        foo-1.0-1.i386.rpm returns foo, 1.0, 1, i386
        1:bar-9-123a.ia64.rpm returns bar, 9, 123a, 1, ia64

    This function is copied from rpmutils.
    Dead code right now as rpm -qa --filesbypkg writes name without version, --queryformat is safer and simpler.
    """

    if filename[-4:] == '.rpm':
        filename = filename[:-4]
    
    archIndex = filename.rfind('.')
    arch = filename[archIndex+1:]

    relIndex = filename[:archIndex].rfind('-')
    rel = filename[relIndex+1:archIndex]

    verIndex = filename[:relIndex].rfind('-')
    ver = filename[verIndex+1:relIndex]

    epochIndex = filename.find(':')
    if epochIndex == -1: 
        epoch = ''
    else:
        epoch = filename[:epochIndex]
    
    name = filename[epochIndex + 1:verIndex]
    return name, ver, rel, epoch, arch

def main():
	"""read VALUE_EXECUTABLES, ignore those from Red Hat and check if they are linked against hal-libs."""
	"""check if any package depends on hal, again ignore Red Hat"""

	def is_elf(file_path):
		"""check for elf magic number, true for match"""
		try:
			f = open(file_path, "rb")
		except IOError:
			#solution_file("can not open " + file_path + "\n")
			return False
		pattern = "\x7FELF"
		try:
			char = f.read(4)
			if char != pattern:
				#solution_file("Not matched " + file_path+ "\n")
				return False
		except IOError:
			#solution_file("can not read " + file_path+ "\n")
			return False
		finally:
			f.close()
		#solution_file("matched " + file_path + "\n")
		return True

	def run_command(cmd):
		"""run external command specified as a list of argv, return status code and output as a list of lines"""
		output = os.tmpfile()
        	try:
        		return_code = subprocess.Popen(cmd,
							stdout=output,
							stderr=subprocess.STDOUT,
							close_fds=True,
							shell=False,
							creationflags=0).wait()
		except:
			output.seek(0)
			log_error(output.read())
			log_error("Could not invoke external command " + str(cmd))
			exit_error()
		output.seek(0)
		return (return_code, output.read())

	def get_package(file_path):
		"""ask rpm if the file belongs to a package, return its name"""
		cmd = ["rpm", "-qf", file_path]
		(return_code, output) = run_command(cmd)
		if return_code !=  0:
			return None
		return output.strip()

	def is_redhat(file_path, redhat_list, filetopkgdict):
		"""check if the executable is from red hat signed package, those are covered elsewhere."""
		"""expects redhat_list - list of redhat package names"""
		"""expects filetopkgdict - dict of file to package mapping"""
		try:
			return filetopkgdict[file_path] in redhat_list
		except KeyError:
			return False
		#return pkg in redhat_list
		
		#pkg = get_package(file_path)
		#if not pkg:
		#	return False
		#(name, ver, rel, epoch, arch) = splitFilename(pkg))

		#for rhlist_item in redhat_list:
		#	if name == rhlist_item:
		#		return True
		#return  False
		#if pkg:
		#	log_error(file_path + "  " + pkg)
		#else:
		#	log_error(file_path + "    SSSSSSSSSSSSSSSSS")
		#return 0

	def debug_write_list(list):
		"""for debugging purposes only, dead code"""
		for item in list:
			solution_file(item.strip()+"\n")

	def ldd(file_path):
		"""run ldd and return output as a list of words if successful"""
        	cmd = ["ldd", file_path]
		(return_code, output) = run_command(cmd)
		if return_code !=  0:
			log_warning("Could not run ldd on " + file_path + "\n")
			return ""
		return output.split()

	def filesbypkg():
		"""run 'rpm -qa --filesbypkg' and make a dict file->package"""
		cmd = ["rpm", "-qa", "--filesbypkg"]
		(return_code, output) = run_command(cmd)
		if return_code != 0:
			log_error(output)
			log_error("Error while invoking rpm -qa --filesbypkg")
			exit_error()
		output = output.split("\n")
		filetopkgdict = {}
		for line in output:
			tmp = line.split()
			if len(tmp) != 2:
				#log_error("Error while parsing output from 'rpm -qa --filesbypkg'")
				#log_error("line is:"+line+":\n")
				#exit_error()
				continue
			filetopkgdict[tmp[1]] = tmp[0]
		#log_error(str(filetopkgdict))
		#exit_error()
		return filetopkgdict

	failed = False
	#create a list of executables
	with open(VALUE_EXECUTABLES, "r") as f:
		filelist = f.readlines()
	filelist = tuple(filelist)
	#strip newlines
	#filelist = map(lambda x: x[:-1], filelist)
	filelist = map(lambda x: x.strip(), filelist)
	#ignore symlinks
	filelist = filter(lambda x: not os.path.islink(x), filelist)
	#ignore pipes and other weirdness
	filelist = filter(os.path.isfile, filelist)
	#take only elf
	filelist = filter(is_elf, filelist)
	#ignore red hat packages, first create dict of all files to packages
	filetopkgdict = filesbypkg()
	#list of redhat packages
	#we need only package names in this list
	#much faster to find members in set than in list
	redhat_list = set(get_dist_native_list())
	filelist = filter(lambda x: not is_redhat(x, redhat_list, filetopkgdict), filelist)
	
	hal_lib_list = ["/usr/lib64/libhal-storage.so.1", "/usr/lib64/libhal-storage.so.1.0.0", "/usr/lib64/libhal.so.1", "/usr/lib64/libhal.so.1.0.0"]
	for file in filelist:
		libs = ldd(file)
		for hal_lib in hal_lib_list:
			if hal_lib in libs:
				log_high_risk(file + " is linked against " + hal_lib)
				failed = True
				break

	#check for rpm packages dependent on hal, hal-libs
	(return_code, output) = run_command(["rpm", "-qa", "--whatrequires", "hal", "hal-libs", "--queryformat", "%{NAME}\n"])
	#log_error(output)
	if return_code == 0:
		for pkg in output.split("\n"):
			pkgstrip = pkg.strip()
			if pkgstrip and (pkgstrip not in redhat_list):
				log_high_risk("Found non-Red Hat rpm package dependent on HAL:" + pkg)
				failed = True
	
	return failed

if __name__ == "__main__":
	if os.geteuid() != 0:
		sys.stdout.write("Need to be root.\n")
		log_slight_risk("The script needs to be run under root account")
		exit_error()
	if main():
		exit_fail()
	else:
		exit_pass()
